// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#include "shell_command_plugin.h"

// This must be included before VersionHelpers.h.
#include <windows.h>

#include <VersionHelpers.h>
#include <flutter/method_channel.h>
#include <flutter/plugin_registrar_windows.h>
#include <flutter/standard_method_codec.h>

#include <map>
#include <memory>
#include <sstream>
#include <iostream>
#include <windows.h>
#include <typeinfo>
#include <list>
#include <string>
#include <iterator>
#include <algorithm>
#include <fstream>
#include <future>
namespace
{

// *** Rename this class to match the windows pluginClass in your pubspec.yaml.
class ShellCommandPlugin  : public flutter::Plugin
{
public:
  static void RegisterWithRegistrar(flutter::PluginRegistrarWindows *registrar);

  ShellCommandPlugin();

  virtual ~ShellCommandPlugin();

private:
  // Called when a method is called on this plugin's channel from Dart.
  void HandleMethodCall(
      const flutter::MethodCall<flutter::EncodableValue> &method_call,
      std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result);
};

// static
void ShellCommandPlugin::RegisterWithRegistrar(
    flutter::PluginRegistrarWindows *registrar)
{
  // *** Replace "sample_plugin" with your plugin's channel name in this call.
  //   OutputDebugString("111");
  auto channel =
      std::make_unique<flutter::MethodChannel<flutter::EncodableValue>>(
          registrar->messenger(), "shell_command",
          &flutter::StandardMethodCodec::GetInstance());

  auto plugin = std::make_unique<ShellCommandPlugin>();
  //  OutputDebugString("111");

  channel->SetMethodCallHandler(
      [plugin_pointer = plugin.get()](const auto &call, auto result) {
        plugin_pointer->HandleMethodCall(call, std::move(result));
      });

  registrar->AddPlugin(std::move(plugin));
}

ShellCommandPlugin::ShellCommandPlugin() {}

ShellCommandPlugin::~ShellCommandPlugin() {}

void ShellCommandPlugin::HandleMethodCall(
    const flutter::MethodCall<flutter::EncodableValue> &method_call,
    std::unique_ptr<flutter::MethodResult<flutter::EncodableValue>> result)
{
  // *** Replace the "getPlatformVersion" check with your plugin's method names.
  // See:
  // https://github.com/flutter/engine/tree/master/shell/platform/common/cpp/client_wrapper/include/flutter
  // and
  // https://github.com/flutter/engine/tree/master/shell/platform/windows/client_wrapper/include/flutter
  // for the relevant Flutter APIs.

  if (method_call.method_name().compare("runCommand") == 0)
  {

    if (!method_call.arguments())
    {
      result->Error("Bad arguments", "No Command Found");
      return;
    }

    std::ostringstream version_stream;
    std::ostringstream fileData;

    const auto &list = method_call.arguments()->ListValue();
    if (list[1].IntValue() == 0)
    {
      version_stream << system(list[0].StringValue().c_str());
      version_stream << ":#";
      version_stream << "null";
    }
    else
    {
      TCHAR path_buf[MAX_PATH];
      DWORD ret_val = GetTempPath(MAX_PATH, path_buf);
      std::ostringstream pathStream;
      if (ret_val > MAX_PATH || (ret_val == 0))
      {
        result->Error("Bad arguments", "Temporary folder not found ");
        return;
      }
      std::string cmd = std::string(list[0].StringValue());
      std::string fileName = std::string("temp.txt");
      std::string s = std::string(cmd) + std::string(" >") + fileName;

     std::vector<std::future<int>> futures;
    futures.push_back(std::async(system, s.c_str()));

    for(auto &e : futures) {
      version_stream <<  e.get();
     }
      version_stream << ":#";
      std::string line;
      std::ifstream myfile(fileName);
      if (myfile.is_open())
      {
        while (getline(myfile, line))
        {
          version_stream << line << "\n";
        }
        myfile.close();
      }
    }
    flutter::EncodableValue response(version_stream.str());
    result->Success(&response);
  }
  else
  {
    result->NotImplemented();
  }
}

} // namespace

void ShellCommandPluginRegisterWithRegistrar(
    FlutterDesktopPluginRegistrarRef registrar)
{

  static auto *plugin_registrars =
      new std::map<FlutterDesktopPluginRegistrarRef,
                   std::unique_ptr<flutter::PluginRegistrarWindows>>;
  auto insert_result = plugin_registrars->emplace(
      registrar, std::make_unique<flutter::PluginRegistrarWindows>(registrar));

 ShellCommandPlugin::RegisterWithRegistrar(insert_result.first->second.get());
}
